/*
 * Routines which deal with the characteristics of the terminal. Uses termcap
 * to be as terminal-independent as possible. 
 *
 */

#include <stdio.h>
#include <ssdef.h>
#include <iodef.h>
#include <brkdef.h>

typedef struct {
           short int status;              /* I/O completion status */
           short int bcount;              /* byte transfer count   */
           int dev_dep_data;              /* device dependant data */
        } iosb;                  /* This is a QIO I/O Status Block */
 
typedef struct {
           short int length;              /* Length of string      */
           char      dtype;               /* Type 14 is a string   */
           char      class;               /* Class 1 is fixed length */
           char      *ptr;                /* Pointer to the string */
        } descr;                 /* once initialized, this is a fixed */
                                 /* length string descriptor */
 
short int __tty_chan = 0;

/*
 * Change terminal to "raw mode", or restore to "normal" mode. "Raw mode"
 * means 1. An outstanding read will complete on receipt of a single
 * keystroke. 2. Input is not echoed.  3. Broadcast not from system is disable 
 */
void 
raw_mode(on)
      int             on;
{
   descr __tty_name;
   void build_descr();
   int i;
   unsigned char __tty_buff;
   iosb __tty_iosb;

      if (on) {
	if (__tty_chan == 0) {
	    build_descr(&__tty_name,"TT:");
	    if (sys$assign(&__tty_name,&__tty_chan,0,0) != SS$_NORMAL) {
		 fprintf(stderr, "Can not initialize terminal\n");
		 exit(1);
            }
            __tty_buff = 79;
            i = sys$qiow(0,__tty_chan,IO$_SETMODE+IO$M_BRDCST,&__tty_iosb,0,0,
                &__tty_buff,8,0,0,0,0);
        }
      } else {
	__tty_buff = 0;
        i = sys$qiow(0,__tty_chan,IO$_SETMODE+IO$M_BRDCST,&__tty_iosb,0,0,
	    &__tty_buff,8,0,0,0,0);
      }
}

void build_descr(descr *p,char *s)    /* Builds a fixed length string */
                                      /* descriptor                   */
{
 
   p->length = strlen(s);
   p->dtype = 14;
   p->class = 1;
   p->ptr = s;
}
 
char getchr()
{
   descr __tty_name;
   char __tty_buff;
   iosb __tty_iosb;
   int i;
 
   i = sys$qiow(0,__tty_chan,IO$_READVBLK+IO$M_NOECHO,&__tty_iosb,0,0,
                &__tty_buff,1,0,0,0,0);
   if (i != SS$_NORMAL) return(0);
   else return(__tty_buff);
}
