/******************************************************************************
 *
 * VMS NEWS (part CMDPOST)
 *
 * Copyright 1990 by Bernd Onasch (E-MAIL: ONASCH@ira.uka.de)
 *
 ******************************************************************************
 *
 * Internal COMMAND file (POST)
 * ----------------------------------------------------------------------------
 *  char *get_text(char *text)
 *        post_header(bool flag)
 *        post_redraw(bool flag,int num,char *txt)
 *   bool post_check_group(char *grp)
 *   bool post_check_dist(char *dist,char *grps)
 *   bool post_check_expire(char *expire)
 *        post_preset(bool flag)
 *        post_send_header(bool flag)
 *        post_send_body(char *filename,bool flag)
 *  char *get_newsgroups()
 *        get_references()
 *        get_managed_file()
 *        get_verify_newsgroups(bool flag)
 *        get_verify_subject(bool flag)
 *        get_verify_distribution(bool flag)
 *        get_verify_reply_to(bool flag)
 *        get_verify_followup_to(bool flag)
 *        get_verify_summary(bool flag)
 *        get_verify_keywords(bool flag)
 *        get_verify_expire(bool flag)
 *        get_verify_control(bool flag)
 *        get_verify_approved(bool flag)
 *        cmd_followup()
 *        cmd_post()
 *        cmd_cancel()
 *
 ******************************************************************************/

#module CMDPOST "CMD V1.24"
#include "header.h"
#include <jpidef.h>

char phdr[NEWS_MAX_PHDR][1024],rhdr[NEWS_MAX_RHDR][1024];
globalref int news_table;

/******************************************************************************/

char *get_text(text)
char *text;
{
  while (*text && *text!=' ' && *text!='\t') text++;
  while (*text && (*text==' ' || *text=='\t')) text++;
  return(text);
}

/*----------------------------------------------------------------------------*/

post_header(flag)
bool flag;
{
  char tmp[1024];

  set_cursor(text_id,4,1);
  put_text(text_id,"From:",0,0,TRUE);
  put_text(text_id,"Newsgroups:",0,0,TRUE);
  put_text(text_id,"Subject:",0,0,TRUE);
  put_text(text_id,"Date:",0,0,TRUE);
  put_text(text_id,"Distribution:",0,0,TRUE);
  put_text(text_id,"Organization:",0,0,TRUE);
  put_text(text_id,"X-News-Reader:",0,0,TRUE);
  if (flag&1) {
    put_text(text_id,"Summary:",0,0,TRUE);
    put_text(text_id,"Reply-To:",0,0,TRUE);
    put_text(text_id,"Followup-To:",0,0,TRUE);
    put_text(text_id,"Keywords:",0,0,TRUE);
    put_text(text_id,"Expires:",0,0,TRUE);
  }
  if (flag&2) {
    put_text(text_id,"In-Reply-To:",0,0,TRUE);
  }
  if (flag&4) {
    put_text(text_id,"Control:",0,0,TRUE);
    put_text(text_id,"Approved:",0,0,TRUE);
  }
  if (userdefault[DEFAULT_NUM_USER_NAME_1].text[0] &&
      userdefault[DEFAULT_NUM_USER_TEXT_1].text[0]) {
    sprintf(tmp,"X-%s: %s",userdefault[DEFAULT_NUM_USER_NAME_1].text,userdefault[DEFAULT_NUM_USER_TEXT_1].text);
    put_text(text_id,tmp,0,0,TRUE);
  }
  if (userdefault[DEFAULT_NUM_USER_NAME_2].text[0] &&
      userdefault[DEFAULT_NUM_USER_TEXT_2].text[0]) {
    sprintf(tmp,"X-%s: %s",userdefault[DEFAULT_NUM_USER_NAME_2].text,userdefault[DEFAULT_NUM_USER_TEXT_2].text);
    put_text(text_id,tmp,0,0,TRUE);
  }
}

/*----------------------------------------------------------------------------*/

post_redraw(flag,num,txt)
bool flag;
int num;
char *txt;
{
  char erg[256],*t;
  int i=0;

  switch (num) {
    case NEWS_PHDR_FROM:	t="From";		break;
    case NEWS_PHDR_NEWSGROUP:	t="Newsgroups";		break;
    case NEWS_PHDR_SUBJECT:	t="Subject";		break;
    case NEWS_PHDR_DATE:	t="Date";		break;
    case NEWS_PHDR_DISTRIBUTION:t="Distribution";	break;
    case NEWS_PHDR_ORGANISATION:t="Organization";	break;
    case NEWS_PHDR_SOFTWARE:	t="X-News-Reader";	break;
    case NEWS_PHDR_SUMMARY:	t="Summary";		break;
    case NEWS_PHDR_REPLY_TO:	t="Reply-To";		break;
    case NEWS_PHDR_FOLLOWUP_TO:	t="Followup-To";	break;
    case NEWS_PHDR_KEYWORDS:	t="Keywords";		break;
    case NEWS_PHDR_EXPIRE:	t="Expires";		break;
    case NEWS_PHDR_IN_REPLY_TO:	t="In-Reply-To";	break;
    case NEWS_PHDR_CONTROL:	t="Control";		break;
    case NEWS_PHDR_APPROVED:	t="Approved";		break;
  }
  if (!(flag&1) && num>=NEWS_PHDR_IN_REPLY_TO) i=5;
  if (flag&4 && num>=NEWS_PHDR_CONTROL) i+=2;
  set_cursor(text_id,4+num-i,1);
  sprintf(erg,"%s: %s",t,txt);
  put_text(text_id,erg,0,0,TRUE);
}

/*----------------------------------------------------------------------------*/

bool post_check_group(grp)
char *grp;
{
  int i=0;

  while (i<groupanz) {
    if (!stricmp(groups[i].name,grp) &&
        (groups[i].mode==NEWS_GRP_POSTING||groups[i].mode==NEWS_GRP_MODERATED))
      return(TRUE);
    i++;
  }
  return(FALSE);
}

/*----------------------------------------------------------------------------*/

bool post_check_dist(dist,grps)
char *dist,*grps;
{

  char grp[256],dst[256],*tp,*tp2;

  if (strchr(dist,'.')) return(FALSE);
  strcpy(tp=grp,grps); sprintf(dst,"%s.",dist);
  while (*tp) {
    tp2=strchr(tp,','); if (tp2!=NULL) *tp2++ ='\0';
    if (!strincmp(tp,dst,strlen(dst)) ||!stricmp(tp,dist)) return(TRUE);
    if (tp2!=NULL) tp=tp2; else *tp='\0';
  }
  strcpy(tp=grp,conf[NEWS_NUM_DISTRIBUTION].text);
  while (*tp) {
    tp2=strchr(tp,','); if (tp2!=NULL) *tp2++ ='\0';
    if (!stricmp(tp,dist)) return(TRUE);
    if (tp2!=NULL) tp=tp2; else *tp='\0';
  }
  return(FALSE);
}

/*----------------------------------------------------------------------------*/

post_check_expire(dat)
char *dat;
{
  char *ptr,tmp[16];

  if (!*dat) return(TRUE);
  ptr=dat;
  if (!isdigit(*ptr)) {
    strncpy(tmp,ptr,3); tmp[3]='\0'; ptr+=3;
    if (strlen(tmp)!=3) return(FALSE);
    if (strcmp(tmp,"Mon") && strcmp(tmp,"Tue") && strcmp(tmp,"Wed") &&
        strcmp(tmp,"Thu") && strcmp(tmp,"Fri") && strcmp(tmp,"Sat") &&
        strcmp(tmp,"Sun")) return(FALSE);
    if (!*ptr || *ptr!=',') return(FALSE); ptr++;
    if (!*ptr || *ptr!=' ') return(FALSE); ptr++;
  }
  if (atoi(ptr)>31) return(FALSE);
  if (atoi(ptr)>9) {
    if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  }
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || *ptr!=' ') return(FALSE); ptr++;
  strncpy(tmp,ptr,3); tmp[3]='\0'; ptr+=3;
  if (strlen(tmp)!=3) return(FALSE);
  if (strcmp(tmp,"Jan") && strcmp(tmp,"Feb") && strcmp(tmp,"Mar") &&
      strcmp(tmp,"Apr") && strcmp(tmp,"May") && strcmp(tmp,"Jun") &&
      strcmp(tmp,"Jul") && strcmp(tmp,"Aug") && strcmp(tmp,"Sep") &&
      strcmp(tmp,"Oct") && strcmp(tmp,"Nov") && strcmp(tmp,"Dec"))
    return(FALSE);
  if (!*ptr || *ptr!=' ') return(FALSE); ptr++;
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || *ptr!=' ') return(FALSE); ptr++;
  if (atoi(ptr)>23) return(FALSE);
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || *ptr!=':') return(FALSE); ptr++;
  if (atoi(ptr)>59) return(FALSE);
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || *ptr!=':') return(FALSE); ptr++;
  if (atoi(ptr)>59) return(FALSE);
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr || !isdigit(*ptr)) return(FALSE); ptr++;
  if (!*ptr) return(TRUE);
  if (*ptr!=' ') return(FALSE); ptr++;
  if (strcmp(ptr,conf[NEWS_NUM_TIMEZONE].text)) return(FALSE);
  return(TRUE);
}

/*----------------------------------------------------------------------------*/

post_preset(flag)
bool flag;
{
  clear_display(text_id);
  set_cursor(text_id,2,1);
  put_text(text_id,"Generating ARTICLE header :-",SMG$M_BOLD,0,TRUE);
  strcpy(phdr[NEWS_PHDR_DATE],get_date());
  sprintf(phdr[NEWS_PHDR_SOFTWARE],"VMS NEWS %s",VERSION);
  sprintf(phdr[NEWS_PHDR_FROM],"%s@%s (%s)",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text,conf[NEWS_NUM_PERSONAL_NAME].text);
  post_header(flag);
  post_redraw(flag,NEWS_PHDR_DATE,phdr[NEWS_PHDR_DATE]);
  post_redraw(flag,NEWS_PHDR_SOFTWARE,phdr[NEWS_PHDR_SOFTWARE]);
  post_redraw(flag,NEWS_PHDR_FROM,phdr[NEWS_PHDR_FROM]);
  post_redraw(flag,NEWS_PHDR_ORGANISATION,conf[NEWS_NUM_ORGANISATION].text);
  if (flag&2) post_redraw(flag,NEWS_PHDR_IN_REPLY_TO,phdr[NEWS_PHDR_IN_REPLY_TO]);
  if (*phdr[NEWS_PHDR_SUBJECT]) post_redraw(flag,NEWS_PHDR_SUBJECT,phdr[NEWS_PHDR_SUBJECT]);
  if (flag&8) {
    post_redraw(flag,NEWS_PHDR_NEWSGROUP,phdr[NEWS_PHDR_NEWSGROUP]);
    post_redraw(flag,NEWS_PHDR_SUBJECT,phdr[NEWS_PHDR_SUBJECT]);
    post_redraw(flag,NEWS_PHDR_DISTRIBUTION,phdr[NEWS_PHDR_DISTRIBUTION]);
    post_redraw(flag,NEWS_PHDR_CONTROL,phdr[NEWS_PHDR_CONTROL]);
    post_redraw(flag,NEWS_PHDR_APPROVED,phdr[NEWS_PHDR_APPROVED]);
  }
}

/*----------------------------------------------------------------------------*/

post_send_header(flag)
bool flag;
{
  char t[1024];

  sprintf(t,"From: %s\r\n",phdr[NEWS_PHDR_FROM]); do_send(t);
  sprintf(t,"Newsgroups: %s\r\n",phdr[NEWS_PHDR_NEWSGROUP]); do_send(t);
  if (*phdr[NEWS_PHDR_SUBJECT]) { sprintf(t,"Subject: %s\r\n",phdr[NEWS_PHDR_SUBJECT]);  do_send(t); }
  if (flag) { sprintf(t,"Date: %s\r\n",phdr[NEWS_PHDR_DATE]); do_send(t); }
  if (*phdr[NEWS_PHDR_REPLY_TO]) { sprintf(t,"Reply-To: %s\r\n",phdr[NEWS_PHDR_REPLY_TO]); do_send(t); }
  if (*phdr[NEWS_PHDR_FOLLOWUP_TO]) { sprintf(t,"Followup-To: %s\r\n",phdr[NEWS_PHDR_FOLLOWUP_TO]); do_send(t); }
  sprintf(t,"Distribution: %s\r\n",phdr[NEWS_PHDR_DISTRIBUTION]); do_send(t);
  if (*conf[NEWS_NUM_ORGANISATION].text) { sprintf(t,"Organization: %s\r\n",conf[NEWS_NUM_ORGANISATION].text); do_send(t); }
  sprintf(t,"X-News-Reader: %s\r\n",phdr[NEWS_PHDR_SOFTWARE]); do_send(t);
  if (*phdr[NEWS_PHDR_KEYWORDS]) { sprintf(t,"Keywords: %s\r\n",phdr[NEWS_PHDR_KEYWORDS]); do_send(t); }
  if (*phdr[NEWS_PHDR_SUMMARY]) { sprintf(t,"Summary: %s\r\n",phdr[NEWS_PHDR_SUMMARY]); do_send(t); }
  if (*phdr[NEWS_PHDR_EXPIRE]) { sprintf(t,"Expires: %s\r\n",phdr[NEWS_PHDR_EXPIRE]); do_send(t); }
  if (*phdr[NEWS_PHDR_CONTROL]) { sprintf(t,"Control: %s\r\n",phdr[NEWS_PHDR_CONTROL]); do_send(t); }
  if (*phdr[NEWS_PHDR_APPROVED]) { sprintf(t,"Approved: %s\r\n",phdr[NEWS_PHDR_APPROVED]); do_send(t); }
  if (*phdr[NEWS_PHDR_IN_REPLY_TO]) { sprintf(t,"In-Reply-To: %s\r\n",phdr[NEWS_PHDR_IN_REPLY_TO]); do_send(t); }
  if (*phdr[NEWS_PHDR_REFERENCES]) { sprintf(t,"References: %s\r\n",phdr[NEWS_PHDR_REFERENCES]); do_send(t); }
  if (userdefault[DEFAULT_NUM_USER_NAME_1].text[0] &&
      userdefault[DEFAULT_NUM_USER_TEXT_1].text[0]) {
    sprintf(t,"X-%s: %s\r\n",userdefault[DEFAULT_NUM_USER_NAME_1].text,userdefault[DEFAULT_NUM_USER_TEXT_1].text);
    do_send(t);
  }
  if (userdefault[DEFAULT_NUM_USER_NAME_2].text[0] &&
      userdefault[DEFAULT_NUM_USER_TEXT_2].text[0]) {
    sprintf(t,"X-%s: %s\r\n",userdefault[DEFAULT_NUM_USER_NAME_2].text,userdefault[DEFAULT_NUM_USER_TEXT_2].text);
    do_send(t);
  }
  do_send("\r\n");
}

/*----------------------------------------------------------------------------*/

post_send_body(filename,flg)
char *filename;
bool flg;
{
  bool flag;
  char tmp[1024],t[1024];
  FILE *fd;

  fd=fopen(filename,"r");
  while (fgets(tmp,1024,fd))
    if (*tmp=='.') { sprintf(t,".%s",tmp); do_send(t); }
    else do_send(tmp);
  fclose(fd);
  if (!flg && *conf[NEWS_NUM_SIGNATURE].text) {
    if (fd=fopen(conf[NEWS_NUM_SIGNATURE].text,"r")) {
      do_send("\r\n");
      while (fgets(tmp,1024,fd))
        if (*tmp=='.') { sprintf(t,".%s",tmp); do_send(t); }
        else do_send(tmp);
    }
    fclose(fd);
  }
  do_send(".\r\n");
  flag=TRUE; while (post_status==NEWS_POST_READY) {
    if (flag) { flag=FALSE; inp_comment("[Article posted, being filed at server]"); }
    wait_sync();
  }
}

/*----------------------------------------------------------------------------*/

char *get_newsgroups()
{
  bool flag;
  uword retlen;
  char t[256],*tp,tmp[256]="\0";
  globalvalue int NEWS$_NOSUCHGRP;

  flag=FALSE; while (!flag) {
    flag=(cli$get_value(c$dsc("NEWSGROUPS"),c$ldsc(t,256),&retlen)!=CLI$_COMMA);
    t[retlen]='\0'; tp=t; if (*tp=='"') { t[strlen(t)-1]='\0'; tp++; }
    if (post_check_group(tp)) { strcat(tmp,tp); strcat(tmp,","); }
    else { strcpy(fao,c$asc(tp)); geterror(fac,NEWS$_NOSUCHGRP,fao); }
  }
  if (strlen(tmp)) tmp[strlen(tmp)-1]='\0';
  tp=tmp; while (*tp) { *tp=locase(*tp); tp++; }
  return (tmp);
}

/*----------------------------------------------------------------------------*/

get_references()
{
  bool flag,flag2;
  uword retlen;
  int i;
  char tmp[1024],fp[1024],t[1024],t1[1024],*tp;
  FILE *fd,*fd2;

  if (!strincmp(articles[articlepos].subject,"Re: ",4) ||
      !strincmp(articles[articlepos].subject,"Re^",3))
    strcpy(phdr[NEWS_PHDR_SUBJECT],articles[articlepos].subject);
  else sprintf(phdr[NEWS_PHDR_SUBJECT],"Re: %s",articles[articlepos].subject);
  i=JPI$_PID;
  c$cks(lib$getjpi(&i,0,0,0,c$ldsc(tmp,10),&retlen)); tmp[retlen]='\0';
  sprintf(fp,"sys$scratch:news_reply_%s.tmp",tmp);
  sprintf(t1,"sys$scratch:news_%s.tmp",tmp);
  sprintf(t,"SAVE %s",fp);
  cli$dcl_parse(c$dsc(t),&news_table,0,0,0); cli$dispatch();
  fd=fopen(fp,"r"); fd2=fopen(t1,"w");
  flag=TRUE; flag2=FALSE;
  while (fgets(tmp,1024,fd)) {
    while (tmp[strlen(tmp)-1]=='\r' || tmp[strlen(tmp)-1]=='\n') tmp[strlen(tmp)-1]='\0';
    if (flag) {
      if (!*tmp) {
        flag=FALSE;
        sprintf(phdr[NEWS_PHDR_IN_REPLY_TO],"%s's message of %s",rhdr[NEWS_RHDR_FROM],rhdr[NEWS_RHDR_DATE]);
        sprintf(phdr[NEWS_PHDR_REFERENCES],"%s %s",rhdr[NEWS_RHDR_REFERENCES],rhdr[NEWS_RHDR_MESSAGE_ID]);
        if (*rhdr[NEWS_RHDR_FOLLOWUP_TO] && stricmp(rhdr[NEWS_RHDR_FOLLOWUP_TO],"poster"))
          strcpy(phdr[NEWS_PHDR_NEWSGROUP],rhdr[NEWS_RHDR_FOLLOWUP_TO]);
        else strcpy(phdr[NEWS_PHDR_NEWSGROUP],rhdr[NEWS_RHDR_NEWSGROUP]);
        fprintf(fd2,"In %s %s writes:\n",rhdr[NEWS_RHDR_MESSAGE_ID],rhdr[NEWS_RHDR_FROM]);
        fprintf(fd2,"\n");
      } else {
        if (!strncmp(tmp,"References:",11)) {
          flag2=TRUE; strcpy(rhdr[NEWS_RHDR_REFERENCES],get_text(tmp));
        } else if ((*tmp==' ' || *tmp=='\t') && flag2) {
          strcat(rhdr[NEWS_RHDR_REFERENCES]," "); strcat(rhdr[NEWS_RHDR_REFERENCES],get_text(tmp));
        } else flag2=FALSE;
        if (!strncmp(tmp,"Date:",5)) strcpy(rhdr[NEWS_RHDR_DATE],get_text(tmp));
        if (!strncmp(tmp,"Subject:",8)) strcpy(rhdr[NEWS_RHDR_SUBJECT],get_text(tmp));
        if (!strncmp(tmp,"From:",5)) {
          strcpy(rhdr[NEWS_RHDR_FROM],get_text(tmp));
          if (tp=strchr(rhdr[NEWS_RHDR_FROM],' ')) *tp='\0';
        }
        if (!strncmp(tmp,"Reply-To:",9)) {
          strcpy(rhdr[NEWS_RHDR_REPLY_TO],get_text(tmp));
          if (tp=strchr(rhdr[NEWS_RHDR_REPLY_TO],' ')) *tp='\0';
        }
        if (!strncmp(tmp,"Newsgroups:",11)) strcpy(rhdr[NEWS_RHDR_NEWSGROUP],get_text(tmp));
        if (!strncmp(tmp,"Followup-To:",12)) strcpy(rhdr[NEWS_RHDR_FOLLOWUP_TO],get_text(tmp));
        if (!strncmp(tmp,"Message-ID:",11)) strcpy(rhdr[NEWS_RHDR_MESSAGE_ID],get_text(tmp));
      }
    } else {
      sprintf(t,"%s %s\n",conf[NEWS_NUM_MARKER].text,tmp); fputs(t,fd2);
    }
  }
  fclose(fd2); fclose(fd); delete(fp);
}

/*----------------------------------------------------------------------------*/

get_managed_file()
{
  int i;
  uword retlen;
  char t1[1024],t2[1024],tmp[1024];
  FILE *fd,*fd2;

  i=JPI$_PID; c$cks(lib$getjpi(&i,0,0,0,c$ldsc(tmp,10),&retlen)); tmp[retlen]='\0';
  sprintf(t1,"sys$scratch:news_%s.tmp",tmp);
  sprintf(t2,"sys$scratch:news_temp_%s.tmp",tmp);
  if (fd=fopen(t1,"r")) {
    if (fd2=fopen(t2,"w")) while (fgets(tmp,1024,fd)) fputs(tmp,fd2);
    fclose(fd2);
  }
  fclose(fd);
  delete(t1);
  sprintf(phdr[NEWS_PHDR_FROM],"%s@%s (%s)",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text,conf[NEWS_NUM_PERSONAL_NAME].text);
  strcpy(phdr[NEWS_PHDR_DATE],get_date());
  if (!*phdr[NEWS_PHDR_NEWSGROUP]) strcpy(phdr[NEWS_PHDR_NEWSGROUP],groups[grouppos].name);
  if (!*phdr[NEWS_PHDR_FOLLOWUP_TO]) strcpy(phdr[NEWS_PHDR_FOLLOWUP_TO],groups[grouppos].name);
  if (!*phdr[NEWS_PHDR_REPLY_TO]) sprintf(phdr[NEWS_PHDR_REPLY_TO],"%s@%s",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text);
  if (!*phdr[NEWS_PHDR_DISTRIBUTION]) strcpy(phdr[NEWS_PHDR_DISTRIBUTION],"local");
  strcpy(phdr[NEWS_PHDR_ORGANISATION],conf[NEWS_NUM_ORGANISATION].text);
  sprintf(phdr[NEWS_PHDR_SOFTWARE],"VMS NEWS %s",VERSION);
  fd=fopen(t1,"w");
  fprintf(fd,"From: %s\n",phdr[NEWS_PHDR_FROM]);
  fprintf(fd,"Date: %s\n",phdr[NEWS_PHDR_DATE]);
  fprintf(fd,"Newsgroups: %s\n",phdr[NEWS_PHDR_NEWSGROUP]);
  fprintf(fd,"Subject: %s\n",phdr[NEWS_PHDR_SUBJECT]);
  fprintf(fd,"Followup-To: %s\n",phdr[NEWS_PHDR_FOLLOWUP_TO]);
  fprintf(fd,"Expires: %s\n",phdr[NEWS_PHDR_EXPIRE]);
  fprintf(fd,"Reply-To: %s\n",phdr[NEWS_PHDR_REPLY_TO]);
  fprintf(fd,"Sender: %s\n",phdr[NEWS_PHDR_FROM]);
  fprintf(fd,"References: %s\n",phdr[NEWS_PHDR_REFERENCES]);
  fprintf(fd,"Control: %s\n",phdr[NEWS_PHDR_CONTROL]);
  fprintf(fd,"Distribution: %s\n",phdr[NEWS_PHDR_DISTRIBUTION]);
  fprintf(fd,"Keywords: %s\n",phdr[NEWS_PHDR_KEYWORDS]);
  fprintf(fd,"Summary: %s\n",phdr[NEWS_PHDR_SUMMARY]);
  fprintf(fd,"Approved: %s\n",phdr[NEWS_PHDR_FROM]);
  fprintf(fd,"Lines: \n");
  fprintf(fd,"Organization: %s\n",conf[NEWS_NUM_ORGANISATION].text);
  fprintf(fd,"X-News-Reader: %s\n",phdr[NEWS_PHDR_SOFTWARE]);
  fprintf(fd,"\n");
  if (fd2=fopen(t2,"r")) while (fgets(tmp,1024,fd2)) fputs(tmp,fd);
  fclose(fd2);
  fclose(fd);
  delete(t2);
}

/*----------------------------------------------------------------------------*/

get_verify_newsgroups(flg)
bool flg;
{
  bool flag;
  char t[256],tmp[256],*tp,*tp2;

  flag=FALSE; while (!flag) {
    if (!*phdr[NEWS_PHDR_NEWSGROUP]) {
      if (userdefault[DEFAULT_NUM_NEWSGROUPS].text[0]) strcpy(phdr[NEWS_PHDR_NEWSGROUP],userdefault[DEFAULT_NUM_NEWSGROUPS].text);
      else strcpy(phdr[NEWS_PHDR_NEWSGROUP],groups[grouppos].name);
    }
    if (!post_check_group(phdr[NEWS_PHDR_NEWSGROUP])) phdr[NEWS_PHDR_NEWSGROUP][0]='\0';
    strcpy(tmp,read_string(kbd_id,"Newsgroups ? ",input_id,phdr[NEWS_PHDR_NEWSGROUP]));
    redraw_lines(paste_id,xROWS-1,2);
    flag=(strlen(tmp)==0); t[0]='\0'; tp=tmp; while (*tp) {
      tp2=strchr(tp,','); if (tp2!=NULL) *tp2++ ='\0';
      if (post_check_group(tp)) { strcat(t,tp); strcat(t,","); }
      if (tp2!=NULL) tp=tp2; else *tp='\0';
    }
    t[strlen(t)-1]='\0'; if (strlen(t)) strcpy(phdr[NEWS_PHDR_NEWSGROUP],t);
    flag=(flag?strlen(phdr[NEWS_PHDR_NEWSGROUP]):strlen(t));
  }
  post_redraw(flg,NEWS_PHDR_NEWSGROUP,phdr[NEWS_PHDR_NEWSGROUP]);
}

/*----------------------------------------------------------------------------*/

get_verify_subject(flg)
bool flg;
{
  if (!*phdr[NEWS_PHDR_SUBJECT]) {
    if (*rhdr[NEWS_RHDR_SUBJECT]) strcpy(phdr[NEWS_PHDR_SUBJECT],rhdr[NEWS_RHDR_SUBJECT]);
    if (userdefault[DEFAULT_NUM_SUBJECT].text[0]) strcpy(phdr[NEWS_PHDR_SUBJECT],userdefault[DEFAULT_NUM_SUBJECT].text);
  }
  if (strlen(phdr[NEWS_PHDR_SUBJECT])>60) phdr[NEWS_PHDR_SUBJECT][60]='\0';
  post_redraw(flg,NEWS_PHDR_SUBJECT,phdr[NEWS_PHDR_SUBJECT]);
  strcpy(phdr[NEWS_PHDR_SUBJECT],read_string(kbd_id,"Subject ? ",input_id,phdr[NEWS_PHDR_SUBJECT]));
  redraw_lines(paste_id,xROWS-1,2);
  post_redraw(flg,NEWS_PHDR_SUBJECT,phdr[NEWS_PHDR_SUBJECT]);
}

/*----------------------------------------------------------------------------*/

get_verify_distribution(flg)
bool flg;
{
  bool flag;
  char t[256],tmp[256],*tp,*tp2;

  if (!*phdr[NEWS_PHDR_DISTRIBUTION]) {
    if (userdefault[DEFAULT_NUM_DISTRIBUTION].text[0]) strcpy(phdr[NEWS_PHDR_DISTRIBUTION],userdefault[DEFAULT_NUM_DISTRIBUTION].text);
    else strcpy(phdr[NEWS_PHDR_DISTRIBUTION],"local");
  }
  flag=FALSE; while (!flag) {
    strcpy(tmp,read_string(kbd_id,"Distribution ? ",input_id,phdr[NEWS_PHDR_DISTRIBUTION]));
    redraw_lines(paste_id,xROWS-1,2);
    flag=(strlen(tmp)==0); t[0]='\0'; tp=tmp; while (*tp) {
      tp2=strchr(tp,','); if (tp2!=NULL) *tp2++ ='\0';
      if (post_check_dist(tp,phdr[NEWS_PHDR_NEWSGROUP])) { strcat(t,tp); strcat(t,","); }
      if (tp2!=NULL) tp=tp2; else *tp='\0';
    }
    t[strlen(t)-1]='\0'; if (strlen(t)) strcpy(phdr[NEWS_PHDR_DISTRIBUTION],t);
    flag=strlen(phdr[NEWS_PHDR_DISTRIBUTION]);
  }
  post_redraw(flg,NEWS_PHDR_DISTRIBUTION,phdr[NEWS_PHDR_DISTRIBUTION]);
}

/*----------------------------------------------------------------------------*/

get_verify_reply_to(flg)
bool flg;
{
  char t[256];

  if (!*phdr[NEWS_PHDR_REPLY_TO]) {
    if (userdefault[DEFAULT_NUM_REPLY_TO].text[0]) strcpy(phdr[NEWS_PHDR_REPLY_TO],userdefault[DEFAULT_NUM_REPLY_TO].text);
    else sprintf(phdr[NEWS_PHDR_REPLY_TO],"%s@%s",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text);
  }
  strcpy(phdr[NEWS_PHDR_REPLY_TO],read_string(kbd_id,"Reply-To ? ",input_id,phdr[NEWS_PHDR_REPLY_TO]));
  redraw_lines(paste_id,xROWS-1,2);
  sprintf(t,"%s@%s",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text);
  if (!stricmp(phdr[NEWS_PHDR_REPLY_TO],t)) *phdr[NEWS_PHDR_REPLY_TO]='\0';
  post_redraw(flg,NEWS_PHDR_REPLY_TO,phdr[NEWS_PHDR_REPLY_TO]);
}

/*----------------------------------------------------------------------------*/

get_verify_followup_to(flg)
bool flg;
{
  bool flag;
  char t[256],*tp,*tp2;

  if (!*phdr[NEWS_PHDR_FOLLOWUP_TO]) {
    if (userdefault[DEFAULT_NUM_FOLLOWUP_TO].text[0]) strcpy(phdr[NEWS_PHDR_FOLLOWUP_TO],userdefault[DEFAULT_NUM_FOLLOWUP_TO].text);
    else strcpy(phdr[NEWS_PHDR_FOLLOWUP_TO],groups[grouppos].name);
  }
  flag=FALSE; while (!flag) {
    strcpy(phdr[NEWS_PHDR_FOLLOWUP_TO],read_string(kbd_id,"Followup-To ? ",input_id,phdr[NEWS_PHDR_FOLLOWUP_TO]));
    redraw_lines(paste_id,xROWS-1,2);
    t[0]='\0'; flag=TRUE; tp=phdr[NEWS_PHDR_FOLLOWUP_TO];
    if (stricmp(phdr[NEWS_PHDR_FOLLOWUP_TO],"poster")) while (*tp) {
      tp2=strchr(tp,','); if (tp2!=NULL) *tp2++ ='\0';
      if (post_check_group(tp)) { strcat(t,tp); strcat(t,","); }
      else flag=FALSE;
      if (tp2!=NULL) tp=tp2; else *tp='\0';
    } else strcpy(t,"poster,");
    t[strlen(t)-1]='\0'; strcpy(phdr[NEWS_PHDR_FOLLOWUP_TO],t);
  }
  if (!strcmp(phdr[NEWS_PHDR_FOLLOWUP_TO],phdr[NEWS_PHDR_NEWSGROUP])) *phdr[NEWS_PHDR_FOLLOWUP_TO]='\0';
  post_redraw(flg,NEWS_PHDR_FOLLOWUP_TO,phdr[NEWS_PHDR_FOLLOWUP_TO]);
}

/*----------------------------------------------------------------------------*/

get_verify_summary(flg)
bool flg;
{
  if (!*phdr[NEWS_PHDR_SUMMARY] && userdefault[DEFAULT_NUM_SUMMARY].text[0])
    strcpy(phdr[NEWS_PHDR_SUMMARY],userdefault[DEFAULT_NUM_SUMMARY].text);
  strcpy(phdr[NEWS_PHDR_SUMMARY],read_string(kbd_id,"Summary ? ",input_id,phdr[NEWS_PHDR_SUMMARY]));
  redraw_lines(paste_id,xROWS-1,2);
  post_redraw(flg,NEWS_PHDR_SUMMARY,phdr[NEWS_PHDR_SUMMARY]);
}

/*----------------------------------------------------------------------------*/

get_verify_keywords(flg)
bool flg;
{
  if (!*phdr[NEWS_PHDR_KEYWORDS] && userdefault[DEFAULT_NUM_KEYWORDS].text[0])
    strcpy(phdr[NEWS_PHDR_KEYWORDS],userdefault[DEFAULT_NUM_KEYWORDS].text);
  strcpy(phdr[NEWS_PHDR_KEYWORDS],read_string(kbd_id,"Keywords ? ",input_id,phdr[NEWS_PHDR_KEYWORDS]));
  redraw_lines(paste_id,xROWS-1,2);
  post_redraw(flg,NEWS_PHDR_KEYWORDS,phdr[NEWS_PHDR_KEYWORDS]);
}

/*----------------------------------------------------------------------------*/

get_verify_expire(flg)
bool flg;
{
  bool flag;

  if (!*phdr[NEWS_PHDR_EXPIRE] && userdefault[DEFAULT_NUM_EXPIRE].text[0])
    strcpy(phdr[NEWS_PHDR_EXPIRE],userdefault[DEFAULT_NUM_EXPIRE].text);
  flag=FALSE; while (!flag) {
    strcpy(phdr[NEWS_PHDR_EXPIRE],read_string(kbd_id,"Expires ? ",input_id,phdr[NEWS_PHDR_EXPIRE]));
    redraw_lines(paste_id,xROWS-1,2);
    flag=post_check_expire(phdr[NEWS_PHDR_EXPIRE]);
  }
  post_redraw(flg,NEWS_PHDR_EXPIRE,phdr[NEWS_PHDR_EXPIRE]);
}

/*----------------------------------------------------------------------------*/

get_verify_control(flg)
bool flg;
{
  bool flag;

  if (!*phdr[NEWS_PHDR_CONTROL] && userdefault[DEFAULT_NUM_CONTROL].text[0])
    strcpy(phdr[NEWS_PHDR_CONTROL],userdefault[DEFAULT_NUM_CONTROL].text);
  flag=FALSE; while (!flag) {
    strcpy(phdr[NEWS_PHDR_CONTROL],read_string(kbd_id,"Control ? ",input_id,phdr[NEWS_PHDR_CONTROL]));
    redraw_lines(paste_id,xROWS-1,2);
    if (!strncmp(phdr[NEWS_PHDR_CONTROL],"cancel ",7)) flag=TRUE;
    if (!strncmp(phdr[NEWS_PHDR_CONTROL],"ihave ",6)) flag=TRUE;
    if (!strncmp(phdr[NEWS_PHDR_CONTROL],"sendme ",7)) flag=TRUE;
    if (!strncmp(phdr[NEWS_PHDR_CONTROL],"newgroup ",9)) flag=TRUE;
    if (!strncmp(phdr[NEWS_PHDR_CONTROL],"rmgroup ",8)) flag=TRUE;
    if (!strcmp(phdr[NEWS_PHDR_CONTROL],"sendsys")) flag=TRUE;
    if (!strcmp(phdr[NEWS_PHDR_CONTROL],"version")) flag=TRUE;
  }
  post_redraw(flg,NEWS_PHDR_CONTROL,phdr[NEWS_PHDR_CONTROL]);
}

/*----------------------------------------------------------------------------*/

get_verify_approved(flg)
bool flg;
{
  char t[256];

  if (!*phdr[NEWS_PHDR_APPROVED]) {
    if (userdefault[DEFAULT_NUM_APPROVED].text[0]) strcpy(phdr[NEWS_PHDR_APPROVED],userdefault[DEFAULT_NUM_APPROVED].text);
    else sprintf(phdr[NEWS_PHDR_APPROVED],"%s@%s",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text);
  }
  strcpy(phdr[NEWS_PHDR_APPROVED],read_string(kbd_id,"Approved ? ",input_id,phdr[NEWS_PHDR_APPROVED]));
  redraw_lines(paste_id,xROWS-1,2);
  sprintf(t,"%s@%s",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text);
  post_redraw(flg,NEWS_PHDR_APPROVED,phdr[NEWS_PHDR_APPROVED]);
}

/*----------------------------------------------------------------------------*/

cmd_followup()
{
  uword retlen;
  char t[256],tmp[256];
  globalvalue int NEWS$_IVFLWUP,NEWS$_IVCMD;
  
  if (noscr) { geterror(fac,NEWS$_IVCMD); return; }
  if (internal_mode==1) { geterror(fac,NEWS$_IVFLWUP); return; }
  cli$get_value(c$dsc("$LINE"),c$ldsc(t,256),&retlen); t[retlen]='\0';
  sprintf(tmp,"POST/%s",t);
  cli$dcl_parse(c$dsc(tmp),&news_table,0,0,0); cli$dispatch();
}

/*----------------------------------------------------------------------------*/

cmd_post()
{
  bool header,reply,managed,control,nodate,dflag,flag,keep=FALSE;
  uword retlen,key=0;
  int i;
  char file[256]="\0",t[1024]="\0",tmp[1024],tmp2[1024],*tp;
  FILE *fd,*fd2;
  globalvalue int NEWS$_NOPSTNG,NEWS$_IVFLWUP,NEWS$_IVCMD,
                  NEWS$_NEWSMGR,NEWS$_ABORT,NEWS$_RESTRICT;
#ifdef NO_MAIL
  globalvalue int NEWS$_NOMAIL;
#endif

  if (restricted) { geterror(fac,NEWS$_RESTRICT); return; }
  for(i=0;i<NEWS_MAX_PHDR;i++) *phdr[i]='\0';
  for(i=0;i<NEWS_MAX_RHDR;i++) *rhdr[i]='\0';
  if (!posting) { geterror(fac,NEWS$_NOPSTNG); return; }
  if (cli$present(c$dsc("Filename"))==CLI$_PRESENT) {
    cli$get_value(c$dsc("Filename"),c$ldsc(file,256),&retlen); file[retlen]='\0'; }
  if (noscr) {
    if (cli$present(c$dsc("Filename"))==CLI$_ABSENT) {
      geterror(fac,NEWS$_IVCMD); return;
    }
    sprintf(tmp,"Really POST file %s (containing HEADER and BODY) ? ",file);
    strcpy(t,read_string(kbd_id,tmp,input_id,0));
    if (!stricmp(t,"YES") || !stricmp(t,"Y")) {
      sprintf(tmp,"%s\r\n",CMD_POST); do_send(tmp); nntp_data.post++;
      flag=TRUE; while (post_status==NEWS_POST_OK) {
        if (flag) { flag=FALSE; inp_comment("[Asking server for post access]"); }
        wait_sync();
      }
      if (post_status==NEWS_POST_READY) post_send_body(file,FALSE);
      post_status=NEWS_POST_OK;
    } else geterror(fac,NEWS$_ABORT);
    return;
  }
  header=(cli$present(c$dsc("HEADER"))==CLI$_PRESENT);
  reply=(cli$present(c$dsc("FOLLOWUP"))==CLI$_PRESENT);
  control=(cli$present(c$dsc("CONTROL"))==CLI$_PRESENT);
  if (control && !newsmgr) { geterror(fac,NEWS$_NEWSMGR); return; }
  managed=(cli$present(c$dsc("MANAGED"))==CLI$_PRESENT);
  if (managed && !newsmgr) { geterror(fac,NEWS$_NEWSMGR); return; }
  nodate=(cli$present(c$dsc("DATE"))==CLI$_NEGATED);
  if (cli$present(c$dsc("NEWSGROUPS"))==CLI$_PRESENT)
    strcpy(phdr[NEWS_PHDR_NEWSGROUP],get_newsgroups());
  if (internal_mode==1 && reply) { geterror(fac,NEWS$_IVFLWUP); return; }
  if (cli$present(c$dsc("SUBJECT"))==CLI$_PRESENT) {
    cli$get_value(c$dsc("SUBJECT"),c$ldsc(t,256),&retlen);
    t[retlen]='\0'; tp=t; if (*tp=='"') { t[strlen(t)-1]='\0'; tp++; }
    strcpy(phdr[NEWS_PHDR_SUBJECT],tp);
  }
  dflag=header+2*reply+4*control;
  if (reply) get_references();
  while (key==0 && stricmp(rhdr[NEWS_RHDR_FOLLOWUP_TO],"poster")) {
    if (managed) {
      get_managed_file();
      key=1;
    } else {
      post_preset(dflag);
      get_verify_newsgroups(dflag);
      get_verify_subject(dflag);
      get_verify_distribution(dflag);
      if (header) {
        get_verify_reply_to(dflag);
        get_verify_followup_to(dflag);
        get_verify_summary(dflag);
        get_verify_keywords(dflag);
        get_verify_expire(dflag);
      }
      if (control) {
        get_verify_control(dflag);
        get_verify_approved(dflag);
      }
      key=read_key(kbd_id,"Is this correct [Y|N|Abort=Ctrl-Z] ? ",input_id);
      if (key==SMG$K_TRM_UPPERCASE_Y || key==SMG$K_TRM_LOWERCASE_Y ||
          key==SMG$K_TRM_CR || key==SMG$K_TRM_ENTER) key=1;
      else if (key!=SMG$K_TRM_CTRLZ) key=0;
    }
  }
  if (key!=SMG$K_TRM_CTRLZ) {
    if (control && !managed) {
      sprintf(tmp,"%s\r\n",CMD_POST); do_send(tmp); nntp_data.post++;
      flag=TRUE; while (post_status==NEWS_POST_OK) {
        if (flag) { flag=FALSE; inp_comment("[Asking server for post access]"); }
        wait_sync();
      }
      if (post_status==NEWS_POST_READY) {
        post_send_header(nodate);
        do_send("\r\n");
        do_send(".\r\n");
        flag=TRUE; while (post_status==NEWS_POST_READY) {
          if (flag) { flag=FALSE; inp_comment("[Control message posted, being filed at server]"); }
          wait_sync();
        }
      }
      post_status=NEWS_POST_OK;
    } else {
      i=JPI$_PID; c$cks(lib$getjpi(&i,0,0,0,c$ldsc(tmp,10),&retlen)); tmp[retlen]='\0';
      sprintf(t,"sys$scratch:news_%s.tmp",tmp);
      if (*file) {
        if (fd=fopen(file,"r")) { fd2=fopen(t,"a+"); while (fgets(tmp,1024,fd)) fputs(tmp,fd2); fclose(fd2); }
        fclose(fd);
      }
      if (managed && *conf[NEWS_NUM_SIGNATURE].text) {
        if (fd=fopen(conf[NEWS_NUM_SIGNATURE].text,"r")) {
          fd2=fopen(t,"a+"); fprintf(fd2,"\n");
          while (fgets(tmp,1024,fd)) fputs(tmp,fd2); fclose(fd2);
        }
        fclose(fd);
      }
#ifdef NO_MAIL
      if (reply && !stricmp(rhdr[NEWS_RHDR_FOLLOWUP_TO],"poster")) {
        geterror(fac,NEWS$_NOMAIL);
      } else {
        i=call_editor(t);
        if (i==1) tp="Post article [Y|Abort=Ctrl-Z] ? ";
        else tp="Do not post this article [Y|Send=S] ? ";
        do key=read_key(kbd_id,tp,input_id);
        while (key!=SMG$K_TRM_UPPERCASE_Y && key!=SMG$K_TRM_LOWERCASE_Y &&
               key!=SMG$K_TRM_UPPERCASE_N && key!=SMG$K_TRM_LOWERCASE_N &&
               key!=SMG$K_TRM_CR          && key!=SMG$K_TRM_ENTER &&
               key!=SMG$K_TRM_UPPERCASE_S && key!=SMG$K_TRM_LOWERCASE_S &&
               key!=SMG$K_TRM_CTRLZ) ;
        if ((i==1 && (key==SMG$K_TRM_UPPERCASE_Y || key==SMG$K_TRM_LOWERCASE_Y ||
                      key==SMG$K_TRM_CR || key==SMG$K_TRM_ENTER)) ||
            (i!=1 && (key==SMG$K_TRM_UPPERCASE_S || key==SMG$K_TRM_LOWERCASE_S))) {
          sprintf(tmp,"%s\r\n",CMD_POST); do_send(tmp); nntp_data.post++;
          flag=TRUE; while (post_status==NEWS_POST_OK) {
            if (flag) { flag=FALSE; inp_comment("[Asking server for post access]"); }
            wait_sync();
          }
          if (post_status==NEWS_POST_READY) {
            if (!managed) post_send_header(nodate);
            post_send_body(t,managed);
          }
          if (post_status!=NEWS_POST_FINISHED) keep=TRUE;
          post_status=NEWS_POST_OK;
        } else keep=TRUE;
      }
#else
      i=call_editor(t);
      if (reply && !stricmp(rhdr[NEWS_RHDR_FOLLOWUP_TO],"poster")) {
        if (i==1) tp="Send article as MAIL [Y|Abort=Ctrl-Z] ? ";
        else tp="Do not send this article as MAIL [Y|Send=S] ? ";
        do key=read_key(kbd_id,tp,input_id);
        while (key!=SMG$K_TRM_UPPERCASE_Y && key!=SMG$K_TRM_LOWERCASE_Y &&
               key!=SMG$K_TRM_UPPERCASE_N && key!=SMG$K_TRM_LOWERCASE_N &&
               key!=SMG$K_TRM_CR          && key!=SMG$K_TRM_ENTER &&
               key!=SMG$K_TRM_UPPERCASE_S && key!=SMG$K_TRM_LOWERCASE_S &&
               key!=SMG$K_TRM_CTRLZ) ;
        if ((i==1 && (key==SMG$K_TRM_UPPERCASE_Y || key==SMG$K_TRM_LOWERCASE_Y ||
                      key==SMG$K_TRM_CR || key==SMG$K_TRM_ENTER)) ||
            (i!=1 && (key==SMG$K_TRM_UPPERCASE_S || key==SMG$K_TRM_LOWERCASE_S)))
          send_mail(t,*rhdr[NEWS_RHDR_REPLY_TO]?rhdr[NEWS_RHDR_REPLY_TO]:rhdr[NEWS_RHDR_FROM],phdr[NEWS_PHDR_SUBJECT],"");
      } else {
        if (i==1) tp="Post article [Y|Abort=Ctrl-Z] ? ";
        else tp="Do not post this article [Y|Send=S] ? ";
        do key=read_key(kbd_id,tp,input_id);
        while (key!=SMG$K_TRM_UPPERCASE_Y && key!=SMG$K_TRM_LOWERCASE_Y &&
               key!=SMG$K_TRM_UPPERCASE_N && key!=SMG$K_TRM_LOWERCASE_N &&
               key!=SMG$K_TRM_UPPERCASE_S && key!=SMG$K_TRM_LOWERCASE_S &&
               key!=SMG$K_TRM_CR          && key!=SMG$K_TRM_ENTER &&
               key!=SMG$K_TRM_CTRLZ) ;
        if ((i==1 && (key==SMG$K_TRM_UPPERCASE_Y || key==SMG$K_TRM_LOWERCASE_Y ||
                      key==SMG$K_TRM_CR || key==SMG$K_TRM_ENTER)) ||
            (i!=1 && (key==SMG$K_TRM_UPPERCASE_S || key==SMG$K_TRM_LOWERCASE_S))) {
          sprintf(tmp,"%s\r\n",CMD_POST); do_send(tmp); nntp_data.post++;
          flag=TRUE; while (post_status==NEWS_POST_OK) {
            if (flag) { flag=FALSE; inp_comment("[Asking server for post access]"); }
            wait_sync();
          }
          if (post_status==NEWS_POST_READY) {
            if (!managed) post_send_header(nodate);
            post_send_body(t,managed);
          }
          if (post_status!=NEWS_POST_FINISHED) keep=TRUE;
          post_status=NEWS_POST_OK;
        } else keep=TRUE;
      }
#endif
    }
  }
  i=JPI$_PID; c$cks(lib$getjpi(&i,0,0,0,c$ldsc(tmp,10),&retlen)); tmp[retlen]='\0';
  sprintf(t,"sys$scratch:news_%s.tmp",tmp);
  if (keep) {
    strcpy(tmp,read_string(kbd_id,"Keep file [enter filename or <CR>] ? ",input_id,0));
    if (*tmp) {
      if (strchr(tmp,':')==NULL && strchr(tmp,'>')==NULL && strchr(tmp,']')==NULL) strcpy(tmp2,"sys$disk:[]");
      else if (strchr(tmp,':')==NULL) strcpy(tmp2,"sys$disk:");
      if (*tmp2) strcat(tmp2,tmp);
      else strcpy(tmp2,tmp);
      rename(t,tmp2);
    }
  }
  delete(t); delete(t);
  switch (internal_mode) {
    case 1: newsgroups_redraw(groupscreen,grouppos,TRUE); break;
    case 2: group_redraw(subjectscreen,articlepos,TRUE); break;
    case 3: article_redraw(articlescreen); break;
  }
}

/******************************************************************************/

cmd_cancel()
{
  bool flag,dflag,nodate;
  int i;
  uword retlen,key;
  char tmp[1024],t[1024];
  globalvalue int NEWS$_IVCMD,NEWS$_IVCNCL,NEWS$_RESTRICT,NEWS$_NOPSTNG,
                  NEWS$_NOTYOUROWN;

  if (noscr) { geterror(fac,NEWS$_IVCMD); return; }
  if (internal_mode==1) { geterror(fac,NEWS$_IVCNCL); return; }
  if (restricted) { geterror(fac,NEWS$_RESTRICT); return; }
  nodate=(cli$present(c$dsc("DATE"))==CLI$_NEGATED);
  for(i=0;i<NEWS_MAX_PHDR;i++) *phdr[i]='\0';
  for(i=0;i<NEWS_MAX_RHDR;i++) *rhdr[i]='\0';
  if (!posting) { geterror(fac,NEWS$_NOPSTNG); return; }
  get_references(); dflag=4;
  strcpy(phdr[NEWS_PHDR_NEWSGROUP],"control");
  sprintf(phdr[NEWS_PHDR_CONTROL],"cancel %s",rhdr[NEWS_RHDR_MESSAGE_ID]);
  sprintf(phdr[NEWS_PHDR_SUBJECT],"cmsg cancel %s",rhdr[NEWS_RHDR_MESSAGE_ID]);
  sprintf(phdr[NEWS_PHDR_FROM],"%s@%s (%s)",conf[NEWS_NUM_USERNAME].text,conf[NEWS_NUM_ADDRESS].text,conf[NEWS_NUM_PERSONAL_NAME].text);
  strcpy(phdr[NEWS_PHDR_APPROVED],phdr[NEWS_PHDR_FROM]);
  strcpy(phdr[NEWS_PHDR_DATE],get_date());
  strcpy(phdr[NEWS_PHDR_DISTRIBUTION],"world");
  if (strcmp(phdr[NEWS_PHDR_FROM],rhdr[NEWS_RHDR_FROM]) && !newsmgr) {
    geterror(fac,NEWS$_NOTYOUROWN); return; }
  dflag=4+8; post_preset(dflag);
  key=read_key(kbd_id,"Really cancel that [Y|N|Abort=Ctrl-Z] ? ",input_id);
  if (key==SMG$K_TRM_UPPERCASE_Y || key==SMG$K_TRM_LOWERCASE_Y ||
      key==SMG$K_TRM_CR || key==SMG$K_TRM_ENTER) {
    sprintf(tmp,"%s\r\n",CMD_POST); do_send(tmp); nntp_data.post++;
    flag=TRUE; while (post_status==NEWS_POST_OK) {
      if (flag) { flag=FALSE; inp_comment("[Asking server for post access]"); }
      wait_sync();
    }
    if (post_status==NEWS_POST_READY) {
      post_send_header(nodate);
      do_send("\r\n");
      do_send(".\r\n");
      flag=TRUE; while (post_status==NEWS_POST_READY) {
        if (flag) { flag=FALSE; inp_comment("[Control message posted, being filed at server]"); }
        wait_sync();
      }
    }
    post_status=NEWS_POST_OK;
  }
  i=JPI$_PID; c$cks(lib$getjpi(&i,0,0,0,c$ldsc(tmp,10),&retlen)); tmp[retlen]='\0';
  sprintf(t,"sys$scratch:news_%s.tmp",tmp);
  delete(t); delete(t);
  switch (internal_mode) {
    case 1: newsgroups_redraw(groupscreen,grouppos,TRUE); break;
    case 2: group_redraw(subjectscreen,articlepos,TRUE); break;
    case 3: article_redraw(articlescreen); break;
  }
}

/******************************************************************************/
