/******************************************************************************
 *
 * VMS NEWS (part STARTUP)
 *
 * Copyright 1990 by Bernd Onasch (E-MAIL: ONASCH@ira.uka.de)
 *
 ******************************************************************************
 *
 * NEWS Startup parameters
 * ----------------------------------------------------------------------------
 * int get_facility()
 *     read_parameters()
 *     cmd_startup()
 *     do_show_license()
 *     do_show_report()
 *     do_conversion(char *filename)
 *     do_exit()
 *     group_search_start(int group)
 *     group_read_parser(int group)
 *     group_save_parser(int group)
 *     group_count_parser(int group)
 *     group_thread_sort()
 *     write_resource(bool flag)
 *     show_newgroups(char *date);
 *     news_start(bool flag)
 *     news_end()
 *
 ******************************************************************************/

#module STARTUP "SYS V1.24"
#include "header.h"

globalref int news_startup,news_table;

/******************************************************************************/

#define SYSMSG_TEXT	1
#define SYSMSG_ID	2
#define SYSMSG_SEVERITY	4
#define SYSMSG_FACILITY	8

int get_facility() {
  int mask;
  uword msglen;
  char msg[256],*ptr;

  /* %SYSTEM-F-ABORT, abort  */
  /* |  6   |1|   5 | (rest) */
  c$cks(sys$getmsg(SS$_ABORT,&msglen,c$ldsc(msg,256),0,0));
  msg[msglen]='\0'; mask=0;
  if (msg[0]=='%') {
    if ((ptr=strchr(msg,','))!=NULL) { mask=SYSMSG_TEXT; *ptr='\0'; }
    switch (strlen(msg)) {
      case  2: mask|=SYSMSG_SEVERITY; break;
      case  6: mask|=SYSMSG_ID; break;
      case  7: mask|=SYSMSG_FACILITY; break;
      case  8: mask|=(SYSMSG_SEVERITY|SYSMSG_ID); break;
      case  9: mask|=(SYSMSG_FACILITY|SYSMSG_SEVERITY); break;
      case 13: mask|=(SYSMSG_FACILITY|SYSMSG_ID); break;
      case 15: mask|=(SYSMSG_FACILITY|SYSMSG_SEVERITY|SYSMSG_ID); break;
    }
  } else if (*msg) mask=1;
  return mask;
}

/******************************************************************************/

read_parameters()
{
  char result[256],tmp[256];
  uword reslen;
  ulong status;

  c$cks(cli$get_value(c$dsc("$VERB"),c$ldsc(result,255),&reslen));
  result[reslen]='\0';
  if (!strcmp(result,"RUN") || !strcmp(result,"MCR")) {
    c$cks(lib$get_foreign(c$ldsc(result,255),0,&reslen,0));
    result[reslen]='\0';
    sprintf(tmp,"NEWS %s",result);
    status=cli$dcl_parse(c$dsc(tmp),&news_startup,0,0,0);
    if (status!=CLI$_NORMAL) sys$exit(SS$_NORMAL);
    cli$dispatch();
  } else if (!strcmp(result,"NEWS")) {
    cmd_startup();
  } else {
#ifdef STRICT_CLI
    printf("-------------------------------------\n");
    printf("Verb: %s\n",result);
    c$cks(cli$get_value(c$dsc("$LINE"),c$ldsc(result,255),&reslen));
    result[reslen]='\0';
    printf("Line: %s\n",result);
    printf("-------------------------------------\n");
    printf("Maybe you used \"$\" instead of \"MCR\" ?\n");
    printf("-------------------------------------\n");
    sys$exit(SS$_NORMAL);
#else
    c$cks(lib$get_foreign(c$ldsc(result,255),0,&reslen,0));
    result[reslen]='\0';
    sprintf(tmp,"NEWS %s",result);
    status=cli$dcl_parse(c$dsc(tmp),&news_startup,0,0,0);
    if (status!=CLI$_NORMAL) sys$exit(SS$_NORMAL);
    cli$dispatch();
#endif
  }
}

/******************************************************************************/

cmd_startup()
{
  char tmp[256];
  uword retlen;

  if (cli$present(c$dsc("HELP"))==CLI$_PRESENT) {
    noscr=TRUE; fac=get_facility()?get_facility():15;
    window_setup();
    cli$dcl_parse(c$dsc("HELP/COMMANDS"),&news_table,0,0,0);
    cli$dispatch();
    smg_rundown();
    sys$exit(SS$_NORMAL);
  }
  if (cli$present(c$dsc("CONVERT"))==CLI$_PRESENT) {
    cli$get_value(c$dsc("CONVERT"),c$ldsc(tmp,256),&retlen);
    tmp[retlen]='\0';
    noscr=TRUE; fac=get_facility()?get_facility():15;
    do_conversion(tmp);
  }
  if (cli$present(c$dsc("LICENSE"))==CLI$_PRESENT) {
    noscr=TRUE; fac=get_facility()?get_facility():15;
    do_show_license();
  }
  if (cli$present(c$dsc("REPORT"))==CLI$_PRESENT) {
    noscr=TRUE; fac=get_facility()?get_facility():15;
    do_show_report();
  }
  if (cli$present(c$dsc("SCREEN"))==CLI$_NEGATED) {
    noscr=TRUE; fac=get_facility()?get_facility():15;
  } else {
    noscr=FALSE; fac=get_facility()?get_facility():9;
  }
  all_article=FALSE;
  if (cli$present(c$dsc("ALL"))==CLI$_PRESENT) all_article=TRUE;
  newgr=TRUE; 
  if (cli$present(c$dsc("NEWGROUPS"))==CLI$_NEGATED) newgr=FALSE;
  if (cli$present(c$dsc("SERVER"))==CLI$_PRESENT) {
    cli$get_value(c$dsc("SERVER"),c$ldsc(tmp,256),&retlen);
    tmp[retlen]='\0';
    add_config(NEWS_NUM_SERVER,tmp,NEWS_MODE_TEMPORARY);
  }
  if (cli$present(c$dsc("PORT"))==CLI$_PRESENT) {
    cli$get_value(c$dsc("PORT"),c$ldsc(tmp,256),&retlen);
    tmp[retlen]='\0';
    add_config(NEWS_NUM_PORT,tmp,NEWS_MODE_TEMPORARY);
    if (!atoi(tmp)) conf[NEWS_NUM_PORT].value=0;
  }
  if (cli$present(c$dsc("REGISTER"))==CLI$_PRESENT) {
    cli$get_value(c$dsc("REGISTER"),c$ldsc(tmp,256),&retlen);
    tmp[retlen]='\0';
    if (access(tmp,ACCESS_READ)==0 || access(tmp,ACCESS_EXISTS)==EOF) strcpy(rcfile,tmp);
  }
#ifdef LOGGING
  sprintf(tmp,"server: %s port: %s (type %s)",
    conf[NEWS_NUM_SERVER].text,conf[NEWS_NUM_PORT].text,
    (conf[NEWS_NUM_PORT].value==0)?"DECnet":"TCP/IP");
  write_logging("Startup");
  write_logging(tmp);
#endif
}

/******************************************************************************/

static char *license[]={
" ",
"+----------------------------------------------------------------------------+",
"| VMS NEWS ####, distribution version                                        |",
"| Copyright 1990,1991,1992 by Bernd Onasch (University of Karlsruhe)         |",
"|                                                                            |",
"| VMS TALK ####                                                              |",
"| Copyright 1992 by Bernd Onasch (University of Karlsruhe)                   |",
"|                                                                            |",
"| Permission is granted to any individual or institution to use, copy,       |",
"| or redistribute this software as long as all of the original files are     |",
"| included unmodified, that it is not sold for profit, and that this copy-   |",
"| right notice is retained. If this software is modifed, then it may be used |",
"| locally but shall never be copied or redistributed at all. The modified    |",
"| version shall not be offered locally as official version or inofficial     |",
"| patch anywhere.                                                            |",
"+----------------------------------------------------------------------------+",
"| Contact: Bernd Onasch                   ONASCH@ira.uka.de                  |",
"|          University of Karlsruhe        ONASCH@informatik.uni-karlsruhe.de |",
"|          Informatik Rechnerabteilung    PSI%45050365300::ONASCH            |",
"|          Kaiserstrasse 12               CCC_ONAS@DULRUU51.BITNET           |",
"|          7500 Karlsruhe (FRG)                                              |",
"| Telephone: +49(721)608-2749                                                |",
"+----------------------------------------------------------------------------+",
" ",
"@"
};

do_show_license()
{
  int i=0,j,cnt=0;
  char *ptr,tmp[16];

  window_setup();
  while(*license[i]!='@') {
    if ((ptr=strstr(license[i],"####"))!=NULL) {
      if (cnt==0) strncpy(tmp,VERSION,4);
      if (cnt==1) strncpy(tmp,TALKVERSION,4);
      cnt++; for (j=0;j<4;j++) *ptr++ =tmp[j];
    }
    printf(" %s\n",license[i++]);
  }
  smg_rundown();
  sys$exit(SS$_NORMAL);
}

/******************************************************************************/

do_show_report()
{
  window_setup();
  printf("\n--------------------------------------------------------------------------\n");
  printf("  Very Mad Students NEWS v%s\n",VERSION);
  printf("                    TALK v%s\n",TALKVERSION);
  printf("--------------------------------------------------------------------------\n");
  printf("  Product release date:    %s\n",MOD_TIME);
  printf("  Local compile time:      %s at %s\n",__DATE__,__TIME__);
  if (!strcmp(DEVICE,"NL:")) {
    printf("  Transport mechanisms:    DECnet object only\n");
  } else {
    printf("  Transport mechanisms:    %s TCP/IP, DECnet object\n",TRANSPORT);
    printf("  TCP/IP device:           \"%s\"\n",DEVICE);
  }
#ifndef CALLABLE_MAIL
#ifdef NO_MAIL
  printf("  Local MAILer support:    callable DECnet MAIL only, no TCP/IP MAIL at all\n");
#else
  printf("  Local MAILer support:    callable DECnet MAIL, spawned TCP/IP MAIL\n");
#endif
#else
  printf("  Local MAILer support:    callable DECnet MAIL and callable TCP/IP MAIL\n");
#endif
#ifdef LSE
  printf("  Callable editors:        TECO,EDT,TPU,LSEDIT\n");
#else
  printf("  Callable editors:        TECO,EDT,TPU\n");
#endif
#ifdef LOGGING
  printf("  Local newsgroup logging: build in\n");
#else
  printf("  Local newsgroup logging: not enabled\n");
#endif
  printf("--------------------------------------------------------------------------\n\n");
  smg_rundown();
  sys$exit(SS$_NORMAL);
}

/******************************************************************************/

do_conversion(filename)
char *filename;
{
  char name[256],tmp[1024],tmp2[1024],*ptr,*ptr2;
  int cnt=0,i;
  FILE *fd1,*fd2;
  globalvalue int NEWS$_RCCONV,NEWS$_RCFILE,NEWS$_RCDONE;

  window_setup();
  strcpy(fao,c$asc(filename));
  if (access(filename,ACCESS_READ)==EOF) { geterror(fac,NEWS$_RCCONV,fao); return; }
  geterror(fac,NEWS$_RCFILE,fao);
  strcpy(name,filename); if ((ptr=strchr(name,";"))!=NULL) *ptr='\0';
  strcat(name,"_NEW");
  for (i=0;i<strlen(name);i++) name[i]=upcase(name[i]);
  fd1=fopen(filename,"r");
  fd2=fopen(name,"w");
  while (fgets(tmp,1023,fd1)) {
    ptr=tmp; while (*ptr) if (*ptr=='\n' || *ptr=='\r') *ptr='\0'; else ptr++;
    ptr=tmp; while (*ptr && *ptr!=' ') ptr++; if (*ptr) ptr--;
    if (*ptr==':') {
      *ptr++ ='\0'; while (*ptr && *ptr==' ') ptr++;
      while ((ptr2=strchr(ptr,','))!=NULL) *ptr2=' ';
      strcpy(tmp2,tmp); strcat(tmp2,"[0-0]:"); strcat(tmp2,ptr);
      fprintf(fd2,"%s\r\n",tmp2);
      cnt++;
    }
  }
  fclose(fd2);
  fclose(fd1);
  strcpy(fao,c$asc(name)); geterror(fac,NEWS$_RCDONE,fao,cnt);
  smg_rundown();
  sys$exit(SS$_NORMAL);
}

/******************************************************************************/

do_exit()
{
#ifdef LOGGING
  char t[256];
#endif

  if (internal_mode<0) internal_mode *= -1;
  internal_mode&=0x07;
  if (print_flag) { print_flag=FALSE; print_file("news_hold.tmp",conf[NEWS_NUM_PRINT].text); }
  if (internal_mode>2) article_end();
  if (internal_mode>1 || oldanz>0) group_end();
  if (internal_mode>0) news_end();
  if (internal_mode==0) {
    if (decnet) decnet_end(0);
    else net_end(0);
    if (clock_id) clock_handle(FALSE);
    smg_rundown();
#ifdef LOGGING
    sprintf(t,"articles (read: %d, posted: %d), groups scanned: %d"
            ,nntp_data.article,nntp_data.post,nntp_data.group);
    write_logging(t);
    write_logging("Shutdown");
#endif
    sys$exit(SS$_NORMAL);
  }
}

/******************************************************************************/

group_search_start(group)
int group;
{
  char *tp,*ptr;
  int val1;

  tp=groups[group].read_art;
  if (tp==NIL) return groups[group].start;
  if (*tp=='<') {
    ptr= ++tp;
    val1=atoi(ptr);
    return val1+1;
  } else return groups[group].start;
}

/******************************************************************************/

group_read_parser(group)
int group;
{
  char delim,*pt,*tp,*ptr;
  int i,j,val1,val2;
  extern bool do_check_equal();

  if (groups[group].read_art!=NIL) {
    tp = (char *)malloc((strlen(groups[group].read_art)+1)*sizeof(char));
    pt = tp;
    strcpy(tp,groups[group].read_art);
    if (*tp=='<') {
      ptr= ++tp; while (*tp && *tp!=' ') tp++; if (*tp) *tp++ ='\0';
      val1=atoi(ptr); for(i=0;i<articleanz;i++)
        if (articles[i].number<=val1) articles[i].unread=FALSE;
    }
    while (*tp) {
      ptr=tp; while (*tp && *tp!=' ' && *tp!='-') tp++;
      delim= *tp; if (*tp) *tp++ ='\0'; val1=val2=atoi(ptr);
      if (delim=='-') {
        ptr=tp; while (*tp && *tp!=' ') tp++; if (*tp) *tp++ ='\0';
        val2=atoi(ptr);
      }
      for(i=0;i<articleanz;i++)
        if (val1<=articles[i].number && articles[i].number<=val2) articles[i].unread=FALSE;
    }
    free(pt);
  }
  if (conf[NEWS_NUM_KILLFILE].value) for(j=0;j<articleanz;j++)
    if (articles[j].unread) for(i=0;i<MAXKILL;i++)
      if (kills[i].group!=NIL)
        if (!strcmp(kills[i].group,"ALL") ||
            do_check_equal(groups[grouppos].name,kills[i].group,FALSE))
          if (kills[i].mode==NEWS_KILL_SUBJECT || articles[j].from!=NIL)
            if (do_check_equal(kills[i].mode==NEWS_KILL_SUBJECT?articles[j].subject:articles[j].from,
                               kills[i].pattern,kills[i].sensitive==NEWS_KILL_SENSITIVE))
              articles[j].unread=FALSE;
}

/******************************************************************************/

group_start_parser(group,art_tmp)
int group;
struct TmpArtList *art_tmp;
{
  char delim,*pt,*tp,*ptr;
  int i,j,val1,val2,num;
  extern bool do_check_equal();

  for (num = groups[group].use_start; num <= groups[group].finish; num++) {
    art_tmp[num-groups[group].use_start].unread = TRUE;
    art_tmp[num-groups[group].use_start].number = num;
  }
  num = groups[group].finish - groups[group].use_start + 1;
  if (groups[group].read_art!=NIL) {
    tp = (char *)malloc((strlen(groups[group].read_art)+1)*sizeof(char));
    pt = tp;
    strcpy(tp,groups[group].read_art);
    if (*tp=='<') {
      ptr= ++tp; while (*tp && *tp!=' ') tp++; if (*tp) *tp++ ='\0';
      val1=atoi(ptr); for(i=0;i<num;i++)
        if (art_tmp[i].number<=val1) art_tmp[i].unread=FALSE;
    }
    while (*tp) {
      ptr=tp; while (*tp && *tp!=' ' && *tp!='-') tp++;
      delim= *tp; if (*tp) *tp++ ='\0'; val1=val2=atoi(ptr);
      if (delim=='-') {
        ptr=tp; while (*tp && *tp!=' ') tp++; if (*tp) *tp++ ='\0';
        val2=atoi(ptr);
      }
      for(i=0;i<num;i++)
        if (val1<=art_tmp[i].number && art_tmp[i].number<=val2) art_tmp[i].unread=FALSE;
    }
    free(pt);
  }
}

/******************************************************************************/

group_save_parser(group)
int group;
{
  char txt[900],txt2[256];
  int i,j;
  struct TmpArtList tmp[MAXARTICLES];
  char *ptr,*p, *pt;
  char delim;
  int val1,val2,article_num;
  extern int compare3();

/*
  j=0; for(i=0;i<articleanz;i++) {
    if (articles[i].unread) j++;
    tmp[i].number=articles[i].number;
    tmp[i].unread=articles[i].unread;
  }
  groups[group].unread=j;
*/
  for(i=groups[group].start;i <= groups[group].finish;i++) {
    tmp[i-groups[group].start].number = i;
    tmp[i-groups[group].start].unread = TRUE;
  }
  j = 0;
  for(i=0; i < articleanz; i++) {
    if (!articles[i].unread)
      tmp[articles[i].number-groups[group].start].unread=articles[i].unread;
    else
      j ++;
  }
  groups[group].unread=j;

/*
  for(i=1; i < articleanz; i++) {
    if(articles[i].number != articles[i-1].number + 1) {
      for(j = articles[i-1].number + 1; j<articles[i].number; j++)
          tmp[j-groups[group].start].unread = FALSE;
    }
  }
*/

  pt = groups[grouppos].read_art;
  if (pt != NIL) {
    while (*pt == ' ') pt++;
    p = (char *)malloc((strlen(pt) + 1)*sizeof(char));
    strcpy(p, pt);
    pt = p;
    if (*p=='<') {
      ptr= ++p; while (*p && *p!=' ') p++; if (*p) *p++ ='\0';
      val1=atoi(ptr); for(i=groups[grouppos].start;i <= val1;i++)
        tmp[i-groups[grouppos].start].unread = FALSE;
    }
    while (*p) {
      ptr=p; while (*p && *p!=' ' && *p!='-') p++;
      delim = *p; if (*p) *p++ ='\0'; val1=val2=atoi(ptr);
      if (delim == '-') {
        ptr=p; while (*p && *p!=' ') p++; if (*p) *p++ ='\0';
        val2=atoi(ptr);
      }
      for(i=val1;i<=val2;i++) {
        if (i<=groups[grouppos].finish && groups[grouppos].start<=i)
            tmp[i-groups[grouppos].start].unread = FALSE;
      }
    }
    free(pt);
  }

  article_num = groups[group].finish - groups[group].start + 1;
  qsort(tmp,article_num,sizeof(struct TmpArtList),compare3);
  if (groups[group].read_art!=NIL) { free(groups[group].read_art); groups[group].read_art=NIL; }
  i=0; while (i<article_num) { if (tmp[i].unread) break; i++; }
  if (i>0) sprintf(txt,"<%d ",tmp[i-1].number); else strcpy(txt,prefix);
  i++;
  while (i<article_num) {
    if (!tmp[i].unread) {
      j=i; while(j<article_num) { if (tmp[j].unread) break; j++; }
      if (j-1>i) {
        sprintf(txt2,"%d-%d ",tmp[i].number,tmp[j-1].number);
        i=j-1;
      } else sprintf(txt2,"%d ",tmp[i].number);
      if (strlen(txt)+strlen(txt2)<=900) strcat(txt,txt2);
      else {
        char tmp[900],*tp,*spc,*minus;
        tp=txt; while (strlen(tp)+strlen(txt2)>900) {
          while (*tp && *tp!=' ') tp++;
          spc=strchr(tp,' '); minus=strchr(tp,'-');
          if (minus!=NULL && spc!=NULL && spc>minus) tp=minus;
          if (*tp) tp++;
        }
        strcpy(tmp,tp); strcpy(txt,tmp); strcat(txt,txt2);
      }
    }
    i++;
  }
  if (*txt) txt[strlen(txt)-1]='\0';
  if (*txt) strcpy(groups[group].read_art=malloc(strlen(txt)+1),txt);
  else groups[group].read_art=NIL;
}

/******************************************************************************/

group_count_parser(group)
int group;
{
  char delim,*tp,*ptr,txt[900];
  int i,val1,val2,cnt;

  if (groups[group].read_art!=NIL) strcpy(tp=txt,groups[group].read_art);
  else tp=NULL;
  cnt=groups[group].oldfinish-groups[group].start+1;
  if (tp==NULL) return(cnt);
  if (*tp=='<') {
    ptr= ++tp; while (*tp && *tp!=' ') tp++; if (*tp) *tp++ ='\0';
    val1=atoi(ptr); cnt-=val1-groups[group].start+1;
  }
  while (*tp) {
    ptr=tp; while (*tp && *tp!=' ' && *tp!='-') tp++;
    delim= *tp; if (*tp) *tp++ ='\0'; val1=val2=atoi(ptr);
    if (delim=='-') {
      ptr=tp; while (*tp && *tp!=' ') tp++; if (*tp) *tp++ ='\0';
      val2=atoi(ptr);
    }
    cnt-=val2-val1+1;
  }
  return(cnt);
}

/******************************************************************************/

#define A_LEN 1024

group_thread_sort()
{
  char *ref,t[A_LEN];
  int tmp,i,j,k,l,m,n;
  static int follow[MAXARTICLES],pred[MAXARTICLES],cnt[MAXARTICLES];
  static struct ArtList buffer;

  for (i=0; i<articleanz; i++) { follow[i]=i+1; pred[i]=i-1; cnt[i]=0; }
  for (i=articleanz-1; i>=0; i--) if (articles[i].references!=NIL && strrchr(articles[i].references,'<')!=NULL) {
    ref=strrchr(articles[i].references,'<');
    for (j=0; j<articleanz && strcmp(ref,articles[j].message_id); j++) ;
    if (j!=articleanz) {
      /******************************************************
       * Pre:  j->i * j -> k      * j -> k                  *
       *            * l -> i -> m * l -> i -> ... -> n -> m *
       *            *             *                         *
       * Post: j->i * j => i => k * j => i -> ... -> n => k *
       *            * l => m      * l => m                  *
       ******************************************************
       * a=>b : needs to be changed                         *
       ******************************************************/
      if (i!=follow[j]) {
        k=follow[j]; l=pred[n=i];
        strcpy(t,"> "); if (!strncmp("> ",articles[n].subject,2) || !strncmp("--",articles[n].subject,2)) strcpy(t,"--");
        strncat(t,articles[n].subject,A_LEN-3); t[A_LEN-1]='\0';
        free(articles[n].subject); strcpy(articles[n].subject=malloc(strlen(t)+1),t);
        tmp=cnt[i]; while (tmp--) {
          n=follow[n];
          strcpy(t,"> "); if (!strncmp("> ",articles[n].subject,2) || !strncmp("--",articles[n].subject,2)) strcpy(t,"--");
          strncat(t,articles[n].subject,A_LEN-3); t[A_LEN-1]='\0';
          free(articles[n].subject); strcpy(articles[n].subject=malloc(strlen(t)+1),t);
        }
        m=follow[n];
        follow[j]=i; pred[i]=j;
        follow[n]=k; if (k<articleanz) pred[k]=n;
        if (l>=0) { follow[l]=m; if (m<articleanz) pred[m]=l;
        } else { if (m<articleanz) pred[m]=l; }
      } else {
        strcpy(t,"> "); if (!strncmp("> ",articles[i].subject,2) || !strncmp("--",articles[i].subject,2)) strcpy(t,"--");
        strncat(t,articles[i].subject,A_LEN-3); t[A_LEN-1]='\0';
        free(articles[i].subject); strcpy(articles[i].subject=malloc(strlen(t)+1),t);
        tmp=cnt[n=i]; while (tmp--) {
          n=follow[n];
          strcpy(t,"> "); if (!strncmp("> ",articles[n].subject,2) || !strncmp("--",articles[n].subject,2)) strcpy(t,"--");
          strncat(t,articles[n].subject,A_LEN-3); t[A_LEN-1]='\0';
          free(articles[n].subject); strcpy(articles[n].subject=malloc(strlen(t)+1),t);
        }
      }
      cnt[j]+=1+cnt[i];
    }
  }
  for (i=0; i<articleanz && pred[i]!= -1; i++) ;
  for (j=0; j<articleanz; j++) if (i!=j) {
    buffer=articles[i]; articles[i]=articles[j]; articles[j]=buffer;
    /***********
     * k->i->l *
     * m->j->n *
     ***********/
    k=pred[i]; l=follow[i];
    m=pred[j]; n=follow[j];
    if (k>=0) follow[k]=j;
    if (m>=0) follow[m]=i;
    if (l<articleanz) pred[l]=j;
    if (n<articleanz) pred[n]=i;
    tmp=pred[i]; pred[i]=pred[j]; pred[j]=tmp;
    tmp=follow[i]; follow[i]=follow[j]; follow[j]=tmp;
    i=follow[j];
  } else i=follow[i];
}

#undef A_LEN

/******************************************************************************/

write_resource(flag)
bool flag;
{
  int i;
  char txt[256],txt2[256];
  FILE *fd;
  globalvalue int NEWS$_RCWRT;

  strcpy(txt,get_file());
  if (!flag) strcat(txt,"_tmp");
  strcpy(txt2,txt); strcat(txt2,"_2"); rename(txt,txt2);
  if(fd=fopen(txt,"w+")) {
    fprintf(fd,"TIME: %s\n",actual_time);
    for(i=0;i<groupanz;i++) if (groups[i].registered)
      if (groups[i].read_art==NIL)
        fprintf(fd,"%s[%d-%d]:\n",groups[i].name,groups[i].start,groups[i].finish);
      else
        fprintf(fd,"%s[%d-%d]:%s\n",groups[i].name,groups[i].start,groups[i].finish,groups[i].read_art);
    fclose(fd);
    delete(txt2);
    return(TRUE);
  } else {
    geterror(fac,NEWS$_RCWRT);
    fclose(fd);
    rename(txt2,txt);
    return(FALSE);
  }
}

/******************************************************************************/

show_newgroups(date)
char *date;
{
  uword key;
  char txt[256];

  sprintf(txt,"%s %s\r\n",CMD_NEWGROUPS,date); do_send(txt);
  nntp_data.newgroups++; do_ngroups=2;
  while (do_ngroups) wait_sync();
  do_queryngroups=NEWS_NG_QUERY;
}

/******************************************************************************/

news_start(restart)
bool restart;
{
  bool flag=0;
  int i,j;
  FILE *fd;
  char txt[1024],txt2[1024],*tp,*ptr;
  extern int compare();
  globalvalue int NEWS$_RCREAD,NEWS$_RCFILE,NEWS$_RCSTYLE;

  inp_comment("[Starting network communication]");
  while (posting==2) wait_sync();
  if (posting==3) { do_netexit=TRUE; internal_mode=0; puts(""); do_exit(); }
  inp_comment("[Request for group LIST]");
  sprintf(txt,"%s\r\n",CMD_LIST); do_send(txt); nntp_data.list++;
  while (!groupanz) wait_sync();
  while (do_newsgroups) wait_sync();
  qsort(groups,groupanz,sizeof(struct GrpList),compare);
  strcpy(actual_time,get_time());
  strcpy(txt,get_file()); if (restart) strcat(txt,"_tmp");
  if (access(txt,ACCESS_EXISTS)==EOF) {
    strcpy(txt2,get_path()); strcat(txt2,"NEWS_DEFAULT.RC");
    if (access(txt2,ACCESS_READ)!=EOF) { strcpy(txt,txt2); flag=1; }
  }
  strcpy(fao,c$asc(txt)); geterror(fac,NEWS$_RCFILE,fao);
  if (fd=fopen(txt,"r")) while (TRUE) {
    if (flag==0) { flag= -1; inp_comment("[Reading user's NEWS resource file]"); }
    if (flag==1) { flag= -2; inp_comment("[Reading system default resource file]"); }
    fgets(txt,1024,fd); if (!strlen(txt)) break;
    if (!strncmp(txt,"TIME: ",6)) { if (newgr) show_newgroups(&txt[6]); continue; }
    tp=txt; while (*tp) { if (*tp=='\n'||*tp=='\r') *tp='\0'; tp++; }
    tp=txt; while (*tp && *tp!='[' && *tp!=':' && *tp!='!') tp++;
    if (*tp && *tp=='[') *tp++ ='\0';
    else { geterror(fac,NEWS$_RCSTYLE); puts(""); do_exit(); }
    for(i=0;i<groupanz;i++) if (!strcmp(txt,groups[i].name)) break;
    if (i<groupanz) {
      groups[i].registered=TRUE;
      ptr=tp; while (*tp && *tp!='-') tp++; *tp++ ='\0'; groups[i].oldstart=atoi(ptr);
      ptr=tp; while (*tp && *tp!=']') tp++; *tp++ ='\0'; groups[i].oldfinish=atoi(ptr);
      if (*tp==':') tp++;
      if (*tp) strcpy(groups[i].read_art=malloc(strlen(tp)+1),tp);
      else groups[i].read_art=NIL;
    }
  }
  else geterror(fac,NEWS$_RCREAD);
  fclose(fd);
  if (flag== -1 && !restart) {
    strcpy(txt2,txt); strcat(txt2,"_old"); rename(txt,txt2);
  }
  read_killfile();
  show_header(" VMS NEWS v%s --- Newsgroups: %d (Posting %s)",VERSION,groupanz,posting?"OK":"none");
  for(i=0;i<groupanz;i++) {
    if (groups[i].finish-groups[i].start<0) {
      groups[i].articles=0;
    } else {
      groups[i].articles=groups[i].finish-groups[i].start+1;
      if (groups[i].articles > MAXARTICLES) {
        groups[i].articles = MAXARTICLES;
        groups[i].start = groups[i].finish - MAXARTICLES + 1;
      }
    }
    if (!groups[i].registered) {
      groups[i].oldstart=0; groups[i].oldfinish=0;
      groups[i].unread=groups[i].articles;
      groups[i].read_art=NIL;
    } else {
      if (groups[i].oldfinish>=groups[i].start)
        j=groups[i].finish-groups[i].oldfinish+group_count_parser(i);
      else j=groups[i].articles;
      groups[i].unread=j;
    }
  }
}

/******************************************************************************/

news_end()
{
  int tim=1;
  char t[256];

  sprintf(t,"%s\r\n",CMD_QUIT); do_send(t); nntp_data.quit++;
  inp_comment("[Disconnecting from server]");
}

/******************************************************************************/
